/* $Id: input.h,v 1.6 2010/09/10 22:55:50 lamonte Exp $
 *
 * input.h
 *
 * Additional acknowledgements, news, and updates to the Qodem Project
 * are available at http://qodem.sourceforge.net .
 *
 * This module is licensed under the GNU General Public License Version 2.
 * Please see the file "COPYING" in this directory for more information
 * about the GNU General Public License Version 2.
 *
 *     Copyright (C) 2009  Kevin Lamonte
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

#ifndef __INPUT_H__
#define __INPUT_H__

/* Curses ----------------------------------------------------------------- */

/* These constants all match the ncurses versions. */

#define Q_COLOR_BLACK	0
#define Q_COLOR_RED	1
#define Q_COLOR_GREEN	2
#define Q_COLOR_YELLOW	3
#define Q_COLOR_BLUE	4
#define Q_COLOR_MAGENTA	5
#define Q_COLOR_CYAN	6
#define Q_COLOR_WHITE	7

#ifndef COLOR_PAIR
typedef unsigned long chtype;
typedef	chtype	attr_t;
#endif

#define Q_ERR     (-1)

#define Q_NCURSES_ATTR_SHIFT       8
#define Q_NCURSES_BITS(mask,shift) ((mask) << ((shift) + Q_NCURSES_ATTR_SHIFT))

#define Q_A_NORMAL	(1UL - 1UL)
#define Q_A_ATTRIBUTES	Q_NCURSES_BITS(~(1UL - 1UL),0)
#define Q_A_CHARTEXT	(Q_NCURSES_BITS(1UL,0) - 1UL)
#define Q_A_COLOR		Q_NCURSES_BITS(((1UL) << 8) - 1UL,0)
#define Q_A_STANDOUT	Q_NCURSES_BITS(1UL,8)
#define Q_A_UNDERLINE	Q_NCURSES_BITS(1UL,9)
#define Q_A_REVERSE	Q_NCURSES_BITS(1UL,10)
#define Q_A_BLINK		Q_NCURSES_BITS(1UL,11)
#define Q_A_DIM		Q_NCURSES_BITS(1UL,12)
#define Q_A_BOLD		Q_NCURSES_BITS(1UL,13)
#define Q_A_ALTCHARSET	Q_NCURSES_BITS(1UL,14)
#define Q_A_INVIS		Q_NCURSES_BITS(1UL,15)
#define Q_A_PROTECT	Q_NCURSES_BITS(1UL,16)
#define Q_A_HORIZONTAL	Q_NCURSES_BITS(1UL,17)
#define Q_A_LEFT		Q_NCURSES_BITS(1UL,18)
#define Q_A_LOW		Q_NCURSES_BITS(1UL,19)
#define Q_A_RIGHT		Q_NCURSES_BITS(1UL,20)
#define Q_A_TOP		Q_NCURSES_BITS(1UL,21)
#define Q_A_VERTICAL	Q_NCURSES_BITS(1UL,22)

#define Q_KEY_CODE_YES	0400		/* A wchar_t contains a key code */
#define Q_KEY_MIN		0401		/* Minimum curses key */
#define Q_KEY_BREAK	0401		/* Break key (unreliable) */
#define Q_KEY_SRESET	0530		/* Soft (partial) reset (unreliable) */
#define Q_KEY_RESET	0531		/* Reset or hard reset (unreliable) */
/*
 * These definitions were generated by /home/user/ncurses/include/MKkey_defs.sh /home/user/ncurses/include/Caps
 */
#define Q_KEY_DOWN	0402		/* down-arrow key */
#define Q_KEY_UP		0403		/* up-arrow key */
#define Q_KEY_LEFT	0404		/* left-arrow key */
#define Q_KEY_RIGHT	0405		/* right-arrow key */
#define Q_KEY_HOME	0406		/* home key */
#define Q_KEY_BACKSPACE	0407		/* backspace key */
#define Q_KEY_F0		0410		/* Function keys.  Space for 64 */
#define Q_KEY_F(n)	(Q_KEY_F0+(n))	/* Value of function key n */
#define Q_KEY_DL		0510		/* delete-line key */
#define Q_KEY_IL		0511		/* insert-line key */
#define Q_KEY_DC		0512		/* delete-character key */
#define Q_KEY_IC		0513		/* insert-character key */
#define Q_KEY_EIC		0514		/* sent by rmir or smir in insert mode */
#define Q_KEY_CLEAR	0515		/* clear-screen or erase key */
#define Q_KEY_EOS		0516		/* clear-to-end-of-screen key */
#define Q_KEY_EOL		0517		/* clear-to-end-of-line key */
#define Q_KEY_SF		0520		/* scroll-forward key */
#define Q_KEY_SR		0521		/* scroll-backward key */
#define Q_KEY_NPAGE	0522		/* next-page key */
#define Q_KEY_PPAGE	0523		/* previous-page key */
#define Q_KEY_STAB	0524		/* set-tab key */
#define Q_KEY_CTAB	0525		/* clear-tab key */
#define Q_KEY_CATAB	0526		/* clear-all-tabs key */
#define Q_KEY_ENTER	0527		/* enter/send key */
#define Q_KEY_PRINT	0532		/* print key */
#define Q_KEY_LL		0533		/* lower-left key (home down) */
#define Q_KEY_A1		0534		/* upper left of keypad */
#define Q_KEY_A3		0535		/* upper right of keypad */
#define Q_KEY_B2		0536		/* center of keypad */
#define Q_KEY_C1		0537		/* lower left of keypad */
#define Q_KEY_C3		0540		/* lower right of keypad */
#define Q_KEY_BTAB	0541		/* back-tab key */
#define Q_KEY_BEG		0542		/* begin key */
#define Q_KEY_CANCEL	0543		/* cancel key */
#define Q_KEY_CLOSE	0544		/* close key */
#define Q_KEY_COMMAND	0545		/* command key */
#define Q_KEY_COPY	0546		/* copy key */
#define Q_KEY_CREATE	0547		/* create key */
#define Q_KEY_END		0550		/* end key */
#define Q_KEY_EXIT	0551		/* exit key */
#define Q_KEY_FIND	0552		/* find key */
#define Q_KEY_HELP	0553		/* help key */
#define Q_KEY_MARK	0554		/* mark key */
#define Q_KEY_MESSAGE	0555		/* message key */
#define Q_KEY_MOVE	0556		/* move key */
#define Q_KEY_NEXT	0557		/* next key */
#define Q_KEY_OPEN	0560		/* open key */
#define Q_KEY_OPTIONS	0561		/* options key */
#define Q_KEY_PREVIOUS	0562		/* previous key */
#define Q_KEY_REDO	0563		/* redo key */
#define Q_KEY_REFERENCE	0564		/* reference key */
#define Q_KEY_REFRESH	0565		/* refresh key */
#define Q_KEY_REPLACE	0566		/* replace key */
#define Q_KEY_RESTART	0567		/* restart key */
#define Q_KEY_RESUME	0570		/* resume key */
#define Q_KEY_SAVE	0571		/* save key */
#define Q_KEY_SBEG	0572		/* shifted begin key */
#define Q_KEY_SCANCEL	0573		/* shifted cancel key */
#define Q_KEY_SCOMMAND	0574		/* shifted command key */
#define Q_KEY_SCOPY	0575		/* shifted copy key */
#define Q_KEY_SCREATE	0576		/* shifted create key */
#define Q_KEY_SDC		0577		/* shifted delete-character key */
#define Q_KEY_SDL		0600		/* shifted delete-line key */
#define Q_KEY_SELECT	0601		/* select key */
#define Q_KEY_SEND	0602		/* shifted end key */
#define Q_KEY_SEOL	0603		/* shifted clear-to-end-of-line key */
#define Q_KEY_SEXIT	0604		/* shifted exit key */
#define Q_KEY_SFIND	0605		/* shifted find key */
#define Q_KEY_SHELP	0606		/* shifted help key */
#define Q_KEY_SHOME	0607		/* shifted home key */
#define Q_KEY_SIC		0610		/* shifted insert-character key */
#define Q_KEY_SLEFT	0611		/* shifted left-arrow key */
#define Q_KEY_SMESSAGE	0612		/* shifted message key */
#define Q_KEY_SMOVE	0613		/* shifted move key */
#define Q_KEY_SNEXT	0614		/* shifted next key */
#define Q_KEY_SOPTIONS	0615		/* shifted options key */
#define Q_KEY_SPREVIOUS	0616		/* shifted previous key */
#define Q_KEY_SPRINT	0617		/* shifted print key */
#define Q_KEY_SREDO	0620		/* shifted redo key */
#define Q_KEY_SREPLACE	0621		/* shifted replace key */
#define Q_KEY_SRIGHT	0622		/* shifted right-arrow key */
#define Q_KEY_SRSUME	0623		/* shifted resume key */
#define Q_KEY_SSAVE	0624		/* shifted save key */
#define Q_KEY_SSUSPEND	0625		/* shifted suspend key */
#define Q_KEY_SUNDO	0626		/* shifted undo key */
#define Q_KEY_SUSPEND	0627		/* suspend key */
#define Q_KEY_UNDO	0630		/* undo key */
#define Q_KEY_MOUSE	0631		/* Mouse event has occurred */
#define Q_KEY_RESIZE	0632		/* Terminal resize event */
#define Q_KEY_EVENT	0633		/* We were interrupted by an event */

#define Q_KEY_MAX		0777		/* Maximum key value is 0633 */

/* Includes --------------------------------------------------------------- */

/* Defines ---------------------------------------------------------------- */

#define q_cursor_off() \
	fprintf(stdout, "\033[?25l"); fflush(stdout)

#define q_cursor_on() \
	fprintf(stdout, "\033[?25h"); fflush(stdout)

/* Globals ---------------------------------------------------------------- */

/* The current rendering color, stored in console.c */
extern attr_t q_current_color;

/* Functions -------------------------------------------------------------- */

/* Main keyboard input function */
extern void qodem_win_getch(void * window, int * keystroke, int * flags, const unsigned int usleep_time);

extern void qodem_getch(int * keystroke, int * flags, const unsigned int usleep_time);

extern void discarding_getch();
extern void set_blocking_input(Q_BOOL block);

#endif /* __INPUT_H__ */
